package jp.co.sds_corp.http.server;

import java.util.ArrayList;
import java.util.List;

import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

public class ContextLoader implements IConfigurable
{
	List<Context> listContext = new ArrayList<Context>();

	@Override
	public void init( IConfiguration config ) throws Exception {
		List<IConfiguration> list = config.listChild();
		for (IConfiguration cfg : list) {
			Context context = createContext(cfg);
			if (context == null) {
				continue;
			}
			listContext.add(context);
		}
	}

	Context createContext(IConfiguration config) throws Exception {
		Context context = null;
		if (config.getName().equals("document")) {
			context = new Document();
		} else
		if (config.getName().equals("webapp")) {
			context = new WebApp();
		} else
		if (config.getName().equals("context")) {
			String cls = config.getAttributes().get("class");
			context = (Context )Class.forName(cls).newInstance();
		} else {
			return null;
		}
		if (context instanceof IConfigurable) {
			((IConfigurable )context).init(config);
		}
		return context;
	}

	public List<Context> listContext() {
		return listContext;
	}

	Context findContext(HttpRequest request) {
		String query = request.getQueryString();
		if (query == null) {
			return null;
		}
		for (Context context : listContext) {
			if (context.match(request)) {
				return context;
			}
		}

		return null;
	}

}
