package jp.co.sds_corp.http.server;

import java.io.ByteArrayOutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class HttpResponse
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	Map<String,String> mapHeader = new HashMap<String,String>();
	String url;
	String protocol;
	int status;
	String message;
	byte[] content;

	public HttpResponse() {
	}

	public static HttpResponse createResponse(IConfiguration config)
		throws Exception
	{
		HttpResponse ret = new HttpResponse();
		String var;
		var = config.getAttributes().get("url", null);
		if (var != null && var.length() > 0) {
			ret.setURL(var);
		}
		var = config.getAttributes().get("status", null);
		if (var != null && var.length() > 0) {
			try {
				ret.setStatus(Integer.valueOf(var));
			} catch (Exception ex) {
			}
		}
		List<IConfiguration> list = config.listChild("header");
		for (IConfiguration hdCfg : list) {
			String name = hdCfg.getAttributes().get("name", null);
			String value = hdCfg.getAttributes().get("value", null);
			if (name == null || name.length() == 0 ||
				value == null || value.length() == 0)
			{
				continue;
			}
			ret.putHeader(name, value);
		}
		IConfiguration contentCfg = config.getChild("content");
		if (contentCfg != null) {
			ret.setContent(contentCfg.get());
		}
		return ret;
	}

	public void putHeaders(Map<String,String> map) {
		mapHeader.putAll(map);
	}
	public void putHeader(String name, String value) {
		mapHeader.put(name, value);
	}
	public String getHeader(String name) {
		String ret = mapHeader.get(name);
		return ret;
	}

	public void setURL(String var) {
		this.url = var;
	}
	public String getURL() {
		return this.url;
	}

	public void setProtocol(String var) {
		this.protocol = var;
	}
	public String getProtocol() {
		return this.protocol;
	}

	public void setStatus(int var) {
		this.status = var;
		switch (status) {
		case HttpServletResponse.SC_ACCEPTED: message = "Accepted"; break;
		case HttpServletResponse.SC_BAD_GATEWAY: message = "Bad Gateway"; break;
		case HttpServletResponse.SC_BAD_REQUEST: message = "Bad Request"; break;
		case HttpServletResponse.SC_CONFLICT: message = "Conflict"; break;
		case HttpServletResponse.SC_CONTINUE: message = "Continue"; break;
		case HttpServletResponse.SC_CREATED: message = "Created"; break;
		case HttpServletResponse.SC_EXPECTATION_FAILED: message = "Expectation Failed"; break;
		case HttpServletResponse.SC_FORBIDDEN: message = "Forbidden"; break;
		case HttpServletResponse.SC_GATEWAY_TIMEOUT: message = "Gateway Timeout"; break;
		case HttpServletResponse.SC_GONE: message = "Gone"; break;
		case HttpServletResponse.SC_HTTP_VERSION_NOT_SUPPORTED: message = "HTTP Version Not Supported"; break;
		case HttpServletResponse.SC_INTERNAL_SERVER_ERROR: message = "Internal Server Error"; break;
		case HttpServletResponse.SC_LENGTH_REQUIRED: message = "Length Required"; break;
		case HttpServletResponse.SC_METHOD_NOT_ALLOWED: message = "Method Not Allowed"; break;
		case HttpServletResponse.SC_MOVED_PERMANENTLY: message = "Moved Permanently"; break;
		case HttpServletResponse.SC_MOVED_TEMPORARILY: message = "Moved Temporary"; break;
		case HttpServletResponse.SC_MULTIPLE_CHOICES: message = "Mulitiple Choices"; break;
		case HttpServletResponse.SC_NO_CONTENT: message = "No Content"; break;
		case HttpServletResponse.SC_NON_AUTHORITATIVE_INFORMATION: message = "NON_AUTHORITATIVE_INFORMATION"; break;
		case HttpServletResponse.SC_NOT_ACCEPTABLE: message = "NOT_ACCEPTABL"; break;
		case HttpServletResponse.SC_NOT_FOUND: message = "Not Found"; break;
		case HttpServletResponse.SC_NOT_IMPLEMENTED: message = "NOT_IMPLEMENTED"; break;
		case HttpServletResponse.SC_NOT_MODIFIED: message = "NOT_MODIFIED"; break;
		case HttpServletResponse.SC_OK: message = "OK"; break;
		case HttpServletResponse.SC_PARTIAL_CONTENT: message = "PARTIAL_CONTENT"; break;
		case HttpServletResponse.SC_PAYMENT_REQUIRED: message = "PAYMENT_REQUIRED"; break;
		case HttpServletResponse.SC_PRECONDITION_FAILED: message = "PRECONDITION_FAILED"; break;
		case HttpServletResponse.SC_PROXY_AUTHENTICATION_REQUIRED: message = "PROXY_AUTHENTICATION_REQUIRED"; break;
		case HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE: message = "REQUEST_ENTITY_TOO_LARGE"; break;
		case HttpServletResponse.SC_REQUEST_TIMEOUT: message = "REQUEST_TIMEOUT"; break;
		case HttpServletResponse.SC_REQUEST_URI_TOO_LONG: message = "REQUEST_URI_TOO_LONG"; break;
		case HttpServletResponse.SC_REQUESTED_RANGE_NOT_SATISFIABLE: message = "REQUESTED_RANGE_NOT_SATISFIABLE"; break;
		case HttpServletResponse.SC_RESET_CONTENT: message = "RESET_CONTENT"; break;
		case HttpServletResponse.SC_SEE_OTHER: message = "SEE_OTHER"; break;
		case HttpServletResponse.SC_SERVICE_UNAVAILABLE: message = "SERVICE_UNAVAILABLE"; break;
		case HttpServletResponse.SC_SWITCHING_PROTOCOLS: message = "SWITCHING_PROTOCOLS"; break;
		case HttpServletResponse.SC_TEMPORARY_REDIRECT: message = "TEMPORARY_REDIRECT:"; break;
		case HttpServletResponse.SC_UNAUTHORIZED: message = "UNAUTHORIZED"; break;
		case HttpServletResponse.SC_UNSUPPORTED_MEDIA_TYPE: message = "UNSUPPORTED_MEDIA_TYPE"; break;
		case HttpServletResponse.SC_USE_PROXY: message = "USE_PROXY"; break;
		default: message = "N/A"; break;
		}
	}
	public int getStatusCode() {
		return this.status;
	}
	public String getStatus() {
		return this.status + " " + this.message;
	}

	public void sendError(int code, String message) {
		this.status = code;
		this.message = message;
	}

	public void setContent(Content var) {
		this.content = var.getBytes();
	}
	public void setContent(String var) {
		this.content = var.getBytes();
	}
	public void setContent(byte[] var) {
		this.content = var;
	}
	public byte[] getContent() {
		return this.content;
	}

	public int getContentLength() {
		if (this.content != null) {
			return this.content.length;
		}
		return 0;
	}

	String getHeader() {
		StringBuilder buf = new StringBuilder();
		buf.append(getProtocol()).append(" ");
		buf.append(getStatus()).append("\n");
		for (String name : mapHeader.keySet()) {
			String value = mapHeader.get(name);
			buf.append(name + ": "+ value + "\n");
		}
		if (getContent() != null) {
			buf.append("Content-Length: " + getContent().length + "\n");
		}
		buf.append("\n");
		return buf.toString();
	}

	public byte[] getBytes() {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		String header = getHeader();
		try {
			bos.write(header.getBytes());
			if (getContent() != null) {
				bos.write(getContent());
			}
		} catch (Exception ex) {
			logger.error(this, "output error", ex);
		}
		return bos.toByteArray();
	}

	public void copyAll(HttpResponse src) {
		mapHeader.clear();
		mapHeader.putAll(src.mapHeader);
		url = src.url;
		protocol = src.protocol;
		status = src.status;
		message = src.message;
		content = src.content;
	}

	public String toString() {
		StringBuilder buf = new StringBuilder();
		buf.append(getHeader());
		buf.append(new String(this.getContent()));
		return buf.toString();
	}

}
