package jp.co.sds_corp.http.server;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import jp.co.sds_corp.util.common.DumpTool;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class RequestParser
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	SocketChannel channel;
	ByteBuffer buf = ByteBuffer.allocate(8192);
	ByteArrayOutputStream bos = new ByteArrayOutputStream();
	Date startAt;
	Date lastReceived;
	DumpTool dump = new DumpTool();

	public RequestParser(SocketChannel channel) {
		this.channel = channel;
	}

	int read() throws IOException {
		int ret = channel.read(buf);
		buf.flip();
		byte[] tmp = new byte[buf.limit()];
		buf.get(tmp);
		bos.write(tmp);
		buf.clear();
		lastReceived = new Date();
		return ret;
	}

	HttpRequest parse() throws Exception {
		boolean endHeader = false;
		String topLine = null;
		Map<String,String> mapHd = new HashMap<String,String>();
		int contLen = 0;
		StringBuilder contBuf = new StringBuilder();
		String req = toString();

/*
		logger.trace("parse:\n" + req);
*/
		BufferedReader rd = new BufferedReader(new StringReader(req));
		int pos = 0;
		for ( ;; ) {
			String line = rd.readLine();
			if (line == null) {
				break;
			}
			pos++;
			if (endHeader) {
				contBuf.append(line).append("\n");
				continue;
			}
			if (pos == 1) {
				topLine = line;
				continue;
			}
			if (line.length() == 0) {
				endHeader = true;
				continue;
			}
			int idx = line.indexOf(": ");
			if (idx >= 0) {
				String name = line.substring(0, idx);
				String value = line.substring(idx + 2);
				mapHd.put(name, value);

				if (name.equals("Content-Length")) {
					contLen = Integer.valueOf(value).intValue();
				}
			}
		}
		rd.close();

/*
		if (logger.isTraceEnable(this)) {
			logger.trace("contLen=" + contLen
				+ " content.length=" + contBuf.length()
				+ " endHeader=" + endHeader + " header=" + mapHd);
		}
*/
		if (!endHeader) {
			return null;
		}
		if (contLen < 0 || contLen < checkContentLength()) {
			return null;
		}

		String[] arrTop = topLine.split(" ");
		if (arrTop.length != 3) {
			throw(new Exception("illegal top line.[" + topLine + "]"));
		}
		this.startAt = new Date();

		HttpRequest request = new HttpRequest();
		request.setMethod(arrTop[0]);
		request.setQueryString(arrTop[1]);
		request.setProtocol(arrTop[2]);
		request.setContentLength(contLen);
		request.setContent(contBuf.toString());
		request.putHeaders(mapHd);
		Map<String,String> params = parseParam(request.getContent());
		if (params != null) {
			request.putParameters(params);
		}
		return request;
	}

	Map<String,String> parseParam(String param) throws IOException {
		Map<String,String> map = new HashMap<String,String>();
		BufferedReader rd = new BufferedReader(new StringReader(param));
		for ( ;; ) {
			String line = rd.readLine();
			if (line == null) {
				break;
			}
			String[] arrPrm = line.split("&");
			for (String prm : arrPrm) {
				String[] kv = prm.split("=");
				if (kv.length == 2) {
					map.put(kv[0], kv[1]);
				} else
				if (kv.length == 1 && prm.endsWith("=")) {
					map.put(kv[0], null);
				}
			}
		}
		rd.close();
		return map;
	}

	int checkContentLength() throws Exception {
		int ret = -1;
		byte[] buf = bos.toByteArray();
		for (int i = 0; i < (buf.length - 4); i++) {
			if (buf[i] == '\r' && buf[i+1] == '\n' &&
				buf[i+2] == '\r' && buf[i+3] == '\n')
			{
				ret = buf.length - i - 4;
				break;
			}
		}
/*
		if (logger.isTraceEnable(this)) {
			logger.trace("len=" + ret + "\n" + dump.dump(buf));
		}
*/
		return ret;
	}

	public SocketChannel getChannel() {
		return channel;
	}

	public String getRemoteAddr() {
		SocketAddress addr = channel.socket().getRemoteSocketAddress();
		if (addr == null) {
			return "N/A";
		}
		return addr.toString();
	}

	public Date getStartAt() {
		return this.startAt;
	}

	public Date getLastReceived() {
		return this.lastReceived;
	}

	@Override
	public String toString() {
		Charset charset = Charset.defaultCharset();
		String ret = null;
		try {
			ret = bos.toString(charset.toString());
		} catch (Exception ex) {
			logger.error(this, "encoding error", ex);
		}
		return ret;
	}

}
