package jp.co.sds_corp.http.server;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class WebApp extends Document
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String webxml;
	Map<String,ServletInfo> mapServlet = new HashMap<String,ServletInfo>();
	List<ServletMapping> listMapping = new ArrayList<ServletMapping>();

	protected WebApp() {
	}

	public void init(IConfiguration config) throws Exception {
		super.init(config);

		webxml = config.getAttributes().get("webxml", null);
		if (webxml != null && webxml.length() > 0) {
			loadWebXml(webxml);
		}
	}

	void loadWebXml(String file) throws Exception {
		logger.trace(this, "load web.xml " + file);
		try {
			DOMConfiguration webxml = new DOMConfiguration(file);
			List<IConfiguration> list;
			list = webxml.listChild("servlet");
			for (IConfiguration config : list) {
				ServletInfo info = new ServletInfo();
				info.init(config);
				mapServlet.put(info.getServletName(), info);
			}
			list = webxml.listChild("servlet-mapping");
			for (IConfiguration config : list) {
				ServletMapping mapping = new ServletMapping();
				mapping.init(config);
				listMapping.add(mapping);

				ServletInfo info = mapServlet.get(mapping.getServletName());
				if (info == null) {
					logger.warn(this, "servlet[" + mapping.getServletName() + "] not found."
						+ mapping);
				}
			}
		} catch (Exception ex) {
			throw(new Exception("load web.xml error", ex));
		}
	}

	@Override
	public void service(HttpRequest request, HttpResponse response)
		throws Exception
	{
		String query = request.getQueryString();
		query = query.replace(getURL(), "");
		for (ServletMapping mapping : listMapping) {
			if(mapping.match(query)) {
				String name = mapping.getServletName();
				ServletInfo info = mapServlet.get(name);
				if (info == null) {
					throw(new Exception("servlet " + name + " not found."));
				}

				logger.trace(this, "call " + info.getServletName() + " "
					+ info.getServletClass());

				response.setStatus(HttpServletResponse.SC_OK);
				Class<?> cls = Class.forName(info.getServletClass());
				HttpServlet servlet = (HttpServlet )cls.newInstance();
				servlet.service(null, null);
			 	return;
			}
		}

		super.service(request, response);
	}

	@Override
	public String printConfig() {
		StringBuilder buf = new StringBuilder();
		buf.append(super.printConfig()).append("\n");

		buf.append("servlet={\n");
		for (Map.Entry<String,ServletInfo> entry : mapServlet.entrySet()) {
			buf.append("\t").append(entry.getKey()).append("=")
				.append(entry.getValue()).append("\n");
		}
		buf.append("}\n");

		buf.append("servlet-mapping={\n");
		for (ServletMapping mapping : listMapping) {
			buf.append("\t").append(mapping.toString()).append("\n");
		}
		buf.append("}");

		return buf.toString();
	}

	@Override
	public String toString() {
		StringBuilder buf = new StringBuilder();
		buf.append(super.toString()).append(" webxml=").append(webxml);
		return buf.toString();
	}

}
